#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "pico/binary_info.h"
#include "comdef.h"
#include "OLED_SSD1306.h"
#include "misakiUTF16.h"
#include "misakiUTF16FontData.h"

#define I2C_LED_ADDR	(SSD1306_ADDR)
#define I2C_SDA_PIN 4
#define I2C_SCL_PIN 5

static uint8_t g_y;
static uint8_t g_x;

static void OLED_SSD1306_init(void);
static void OLED_SSD1306_clear(void);
static char* OLED_SSD1306_writeChar(char *data);
static void OLED_SSD1306_newLine(void);
static void OLED_SSD1306_println(char *data);

int main(void)
{
	// This example will use I2C0 on the default SDA and SCL pins (4, 5 on a Pico)
	i2c_init(i2c_default, 100 * 1000);
	gpio_set_function(I2C_SDA_PIN, GPIO_FUNC_I2C);
	gpio_set_function(I2C_SCL_PIN, GPIO_FUNC_I2C);
	gpio_pull_up(I2C_SDA_PIN);
	gpio_pull_up(I2C_SCL_PIN);
	// Make the I2C pins available to picotool
	bi_decl(bi_2pins_with_func(I2C_SDA_PIN, I2C_SCL_PIN, GPIO_FUNC_I2C));
	
	OLED_SSD1306_init();
	
	OLED_SSD1306_clear();
	OLED_SSD1306_println("Hello world.\n");
	OLED_SSD1306_println("こんにちは、世界！\n");
	OLED_SSD1306_println("よろしくお願いいたします！！\n");
	
	while (1U)
	{
		;
	}
	
	return 0;
}

static void OLED_SSD1306_init(void)
{
	uint8_t buf[29];
	
	buf[0] = SSD1306_CONTROL_CMD_STREAM;				/* コントロールバイト */
	
	/* Display off */
	buf[1] = SSD1306_DISPLAY_OFF;
	
	/* Set MUX Ratio A8h, 3Fh*/
	buf[2] = SSD1306_SET_MULTIPLEX_RATIO;
	buf[3] =0x3F;
	
	/* Set Display Offset D3h, 00h */
	buf[4] = SSD1306_SET_DISPLAY_OFFSET;
	buf[5] = 0x00;
	
	/* Set Display Start Line 40h */
	buf[6] = SSD1306_SET_DISPLAY_START_LINE;
	
	/* Set Segment re-map A0h/A1h */
	buf[7] = SSD1306_SET_SEGMENT_REMAP_LOW;
	
	/* Set COM Output Scan Direction C0h/C8h */
	buf[8] = SSD1306_SET_COM_SCAN_INC;
	
	/* Set COM Pins hardware configuration DAh, 02*/
	buf[9] = SSD1306_SET_COM_PIN_MAP;
	buf[10] = 0x12;
	
	/* Set Contrast Control 81h, 7Fh */
	buf[11] = SSD1306_SET_CONTRAST;
	buf[12] = 0x7F;
	
	/* Disable Entire Display On A4h */
	buf[13] = SSD1306_DISPLAY_ALL_ON_RESUME;
	
	/* Set Normal Display A6h */
	buf[14] = SSD1306_NORMAL_DISPLAY;
	
	/* Set Osc Frequency D5h, 80h */
	buf[15] = SSD1306_SET_DISPLAY_CLK_DIV;
	buf[16] = 0x80;	
	
	buf[17] = SSD1306_SET_MEMORY_ADDR_MODE;
	buf[18] = 0x10;
	
	buf[19] = SSD1306_SET_COLUMN_RANGE;
	buf[20] = 0x00;
	buf[21] = 0x7F;
	
	buf[22] = SSD1306_SET_PAGE_RANGE;
	buf[23] = 0x00;
	buf[24] = 0x07;
	
	buf[25] = SSD1306_DEACTIVATE_SCROLL;
	
	/* Enable charge pump regulator 8Dh, 14h */
	buf[26] = SSD1306_SET_CHARGE_PUMP;
	buf[27] = 0x14;
	
	/* Display On AFh */
	buf[28] = SSD1306_DISPLAY_ON;
	
	(void)i2c_write_blocking (i2c_default, I2C_LED_ADDR, buf, 29, false);
	
	return;
}

static void OLED_SSD1306_clear(void)
{
	uint8_t i,j;
	uint8_t buf[9];
	
	for(i = 0; i < 8; i++) {
		buf[0] = SSD1306_CONTROL_CMD_STREAM;			/* コントロールバイト */
		buf[1] = SSD1306_SET_PAGE_START_ADDRESS | i;	/* set page start address(B0～B7) */
		buf[2] = SSD1306_SET_COLUMN_RANGE;				/* set Column Address */
		buf[3] = 0x00;									/* Column Start Address(0-127) */
		buf[4] = 0x7F;									/* Column Stop Address(0-127) */
		
		(void)i2c_write_blocking (i2c_default, I2C_LED_ADDR, buf, 5, false);
		
		buf[0] = SSD1306_CONTROL_DATA_STREAM;			/*	コントロールバイト */
		for(j = 0; j < 8; j++) {
			buf[1 + j] = 0x00;
		}
		for(j = 0; j < 16; j++) {
			(void)i2c_write_blocking (i2c_default, I2C_LED_ADDR, buf, 9, false);
		}
	}
	g_x = 0;
	g_y = 0;
	
	return;
}

static char* OLED_SSD1306_writeChar(char *data)
{
	uint8_t buf[9];
	
	if (*data == 0x0a) {
		OLED_SSD1306_newLine();
		data += 1;
	} else if ((*data == 0x5c) && (*(data + 1) == 0x6e)) {
		OLED_SSD1306_newLine();
		data += 2;
	} else if ((g_y < 8) && (g_x < 16)) {
		/* コマンド */
		buf[0] = SSD1306_CONTROL_CMD_STREAM;			/* コントロールバイト */
		buf[1] = SSD1306_SET_PAGE_START_ADDRESS | g_y;	/* ページ指定, 垂直位置 */
		buf[2] = SSD1306_SET_COLUMN_RANGE;				/* サブアドレス */
		buf[3] = g_x << 3;								/* 水平位置 */
		buf[4] = (g_x << 3) + 7;						/* 折り返し */
		(void)i2c_write_blocking (i2c_default, I2C_LED_ADDR, buf, 5, false);
		
		/* データ */
		buf[0] = SSD1306_CONTROL_DATA_STREAM;			/* コントロールバイト */
		data = getFontData(&buf[1], data, false);
		
		(void)i2c_write_blocking (i2c_default, I2C_LED_ADDR, buf, 9, false);
		g_x++;
	} else {
		;
	}
	
	return data;
}

static void OLED_SSD1306_newLine(void)
{
	g_x = 0;
	if(g_y < 8) {
		g_y++;
	} else {
		g_y = 0;
	}
	
	return;
}

static void OLED_SSD1306_println(char *data)
{	
	do {
		data = OLED_SSD1306_writeChar(data);
	} while(*data != '\0');
	
	return;
}
